/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago4q.importer;

import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.dao.AbstractHibernateDAO;
import it.eng.spago4q.metadata.S4qEScript;

import org.apache.log4j.Logger;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Restrictions;

public class EScriptImportHibImpl extends AbstractHibernateDAO implements
		EScriptImport {

	private static Logger logger = Logger.getLogger(EScriptImportHibImpl.class);

	public void importData(final S4qEScript script, final boolean overwrite)
			throws EMFUserError {
		Session session = null;
		Transaction tx = null;
		try {
			session = getCurrentSession();
			tx = session.beginTransaction();
			importScript(session, script, overwrite);

			session.flush();
			tx.commit();
		} catch (HibernateException he) {
			logException(he);
			if (tx != null) {
				tx.rollback();
			}
			throw new EMFUserError(EMFErrorSeverity.ERROR, 101);
		} finally {
			if (session != null && session.isOpen()) {
				session.close();
			}
			logger.debug("OUT");
		}
	}

	private S4qEScript importScript(final Session session, final S4qEScript script,
			final boolean overwrite) {
		S4qEScript toReturn = null;
		S4qEScript oldScript = findScript(session, script);
		// find script into the DB
		if (oldScript != null) {
			toReturn = oldScript;
			if (overwrite) {
				// fill the data of the DB script with the data of the
				// script imported
				toReturn.setName(script.getName());
				toReturn.setDescription(script.getDescription());
				toReturn.setScript(script.getScript());
				toReturn.setScriptType(script.getScriptType());
			}
		} else {
			// the source type isn't in the DB
			Integer idScript = (Integer) session.save(script);
			toReturn = (S4qEScript) session.get(S4qEScript.class, idScript);
		}
		return toReturn;
	}

	private S4qEScript findScript(final Session session, final S4qEScript script) {
		S4qEScript toReturn = null;
		Criteria crit = session.createCriteria(S4qEScript.class);
		crit.add(Restrictions.eq("name", script.getName()));
		toReturn = (S4qEScript) crit.uniqueResult();
		return toReturn;
	}
}
