/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago4q.extractors.tablehandlers.oracle;

import java.util.Iterator;
import java.util.List;

import it.eng.spago.dbaccess.SQLStatements;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.bo.InterfaceField;
import it.eng.spago4q.dao.DAOFactory;
import it.eng.spago4q.dao.IES4QFKDAO;
import it.eng.spago4q.extractors.tablehandlers.AbstractSqlCodeGenerator;
import it.eng.spago4q.extractors.utilities.ExtractorConstants;

import org.apache.log4j.Logger;

public class OracleSqlCodeGenerator extends AbstractSqlCodeGenerator {
    private static final String ORACLE = "ORACLE";
    
    private static final String SEQUENCE_NAME_PLACE_HOLDER = "@sequenceName";
    private static final String TRIGGER_NAME_PLACE_HOLDER = "@triggerName";
    
    private static Logger logger = Logger
            .getLogger(OracleSqlCodeGenerator.class);
    
    private static String createSequenceName(final String tableName) {
        return tableName + "_SEQ";
    }
    
    private static String createTriggerName(final String tableName) {
        return "TRG_" + tableName;
    }
    
    public static String generateCreateSequenceQuery(final String tableName) {
        String query = SQLStatements
                .getStatement("ORACLE_CREATE_TABLE_SEQUENCE");
        query = query.replace(SEQUENCE_NAME_PLACE_HOLDER,
                createSequenceName(tableName));
        return query;
    }
    
    public static String generateCreateTriggerQuery(final String tableName) {
        String query = SQLStatements
                .getStatement("ORACLE_CREATE_TABLE_TRIGGER");
        query = query.replace(SEQUENCE_NAME_PLACE_HOLDER,
                createSequenceName(tableName));
        query = query.replace(TRIGGER_NAME_PLACE_HOLDER,
                createTriggerName(tableName));
        query = query.replace(TABLE_NAME_PLACE_HOLDER, tableName);
        return query;
    }
    
    @Override
    public String generateDropTableQuery(final String tableName) {
        String query = SQLStatements.getStatement("ORACLE_DROP_TABLE");
        query = query.replace(TABLE_NAME_PLACE_HOLDER, tableName);
        return query;
    }
    
    @Override
    public String generateCreateTableQuery(final String tableName,
            final List interfaceFieldList) throws EMFUserError {
        String columnNameType = "";
        for (Iterator iterator = interfaceFieldList.iterator(); iterator
                .hasNext();) {
            InterfaceField interfaceField = (InterfaceField) iterator.next();

            if (!isValidColumName(interfaceField.getName())) {
                throw new EMFUserError(EMFErrorSeverity.ERROR, 10035);
            }

            columnNameType += ", " + interfaceField.getName() + " ";
            IES4QFKDAO eDomainValueDAO = (IES4QFKDAO) DAOFactory
                    .getDAO("EDomainValueDAO");
            List domainValueList = eDomainValueDAO.loadObjectList(
                    interfaceField.getId(), null, null);

            if (domainValueList == null
                    || (domainValueList.isEmpty())) {
                columnNameType += typeConverter(interfaceField.getFieldType());
            } else {
                columnNameType += typeConverter(ExtractorConstants.INTEGER);
            }
        }

        String commandString = SQLStatements
                .getStatement("ORACLE_CREATE_TABLE");
        commandString = commandString.replace(TABLE_NAME_PLACE_HOLDER,
                tableName);
        commandString = commandString.replace(COLUMN_NAME_TYPE_PLACE_HOLDER,
                columnNameType);

        logger.debug("Return this Create query:" + commandString);
        return commandString;
    }
    
    public static String generateDropSequenceQuery(final String tableName) {
        String query = SQLStatements
                .getStatement("ORACLE_DROP_TABLE_SEQUENCE");
        query = query.replace(SEQUENCE_NAME_PLACE_HOLDER,
                createSequenceName(tableName));
        return query;
    }
    
    public static String generateDropTriggerQuery(final String tableName) {
        String query = SQLStatements
                .getStatement("ORACLE_DROP_TABLE_TRIGGER");
        query = query.replace(TRIGGER_NAME_PLACE_HOLDER,
                createTriggerName(tableName));
        return query;
    }
    
    @Override
    public String getDBConfEnvelopeName() {
        return ORACLE;
    }
    
}
