/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago4q.testUtils;

import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.configuration.FileCreatorConfiguration;
import it.eng.spago4q.dao.DAOFactory;
import it.eng.spago4q.dao.IES4QDAO;

import java.io.FileInputStream;
import java.net.URL;
import java.sql.Connection;
import java.sql.DriverManager;

import org.dbunit.database.DatabaseConfig;
import org.dbunit.database.DatabaseConnection;
import org.dbunit.database.IDatabaseConnection;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.xml.FlatXmlDataSet;
import org.dbunit.ext.hsqldb.HsqldbDataTypeFactory;
import org.dbunit.operation.DatabaseOperation;

import junit.framework.TestCase;

public abstract class AbstractDBTestCase extends TestCase {

	public AbstractDBTestCase(final String value) {
		super(value);
	}

	private Connection jdbcConnection;
	
	private void setConnection (final Connection connection) {
		this.jdbcConnection = connection;
	}
	
	protected void setUp() throws Exception {
		super.setUp();
		ClassLoader loader = this.getClass().getClassLoader();
		URL xmlUrl = loader.getResource("master.xml");
		System.setProperty("AF_CONFIG_FILE", xmlUrl.getPath());
        ConfigSingleton.setConfigurationCreation(new FileCreatorConfiguration(ConfigSingleton.getRootPath())); 
		IES4QDAO foo = (IES4QDAO) DAOFactory.getDAO("ESourceTypeDAO");
		
		foo.loadObjectList(null, null);

		// initialize your database connection here
		setConnection( DriverManager.getConnection(
				"jdbc:hsqldb:mem:/spagobi", "sa", ""));

		IDatabaseConnection connection = new DatabaseConnection(jdbcConnection);
		DatabaseConfig config = connection.getConfig();
		config.setProperty(DatabaseConfig.PROPERTY_DATATYPE_FACTORY,
				new HsqldbDataTypeFactory());

		// initialize your dataset here
		IDataSet dataSet = new FlatXmlDataSet(new FileInputStream(
				getFlatFile()));

		try {
			DatabaseOperation.CLEAN_INSERT.execute(connection, dataSet);
		} finally {
			connection.close();
		}
	}

	protected abstract String getFlatFile();

}
